(function ($, Drupal, once) {
  Drupal.behaviors.layoutParagraphsSplit = {
    attach: function attach(context) {
      const behavior = this;

      // Each time a component is updated...
      once('lps-component-update', '[data-lpb-id]').forEach((el) => $(el).on('lpb-component:update.lpb', () => {
        if (window._layoutParagraphsSplit?.nextStep === 'duplicate') {
          // A paragraph has been cut, we need to create the second
          // paragraph and put the second HTML text part in it.
          behavior.throbber = $(Drupal.theme('ajaxProgressIndicatorFullscreen'));
          $('body').append(behavior.throbber);
          // Wait for layout paragraphs builder save, reorder,...
          document.body.dataset.waitForAjax = true;
        }
        if (window._layoutParagraphsSplit?.nextStep === 'create_in_between') {
          // The second paragraph has been filled and save button clicked, we
          // have to open the create dialog.
          // Wait for layout paragraphs builder save, reorder,...
          document.body.dataset.waitForAjax = true;
        }
      }));

      // Each time an AJAX request starts...
      if (once('lps-ajax-start', 'body').length) {
        $(document).ajaxStart(function () {
          if (window._layoutParagraphsSplit && document.body.dataset.waitForAjax) {
            document.body.dataset.waitForAjax = false;
            document.body.dataset.waitForAjaxStop = true;
          }
        });
      }

      // Each time an AJAX request ends...
      if (once('lps-ajax-stop', 'body').length) {
        $(document).ajaxStop(function () {
          if (window._layoutParagraphsSplit && document.body.dataset.waitForAjaxStop) {
            document.body.dataset.waitForAjaxStop = false;

            const splitUuid = window._layoutParagraphsSplit.uuidToSplit;
            switch (window._layoutParagraphsSplit.nextStep) {
              case 'duplicate':
                // Duplicate the paragraph.
                behavior.getDuplicateButton(splitUuid).click();
                document.body.dataset.waitForAjax = true;
                window._layoutParagraphsSplit.nextStep = 'edit_duplicate';
                break;
              case 'edit_duplicate':
                duplicateUuid = behavior.getNextComponentUuid(splitUuid);
                behavior.getEditButton(duplicateUuid).click();
                document.body.dataset.waitForAjax = true;
                window._layoutParagraphsSplit.nextStep = 'fill_and_save_duplicate';
                // Next step is handled in the CKE5 plugin.
                break;
              case 'create_in_between':
                behavior.getAddAfterButton(splitUuid).click();
                document.body.dataset.waitForAjax = true;
                window._layoutParagraphsSplit.nextStep = 'end';
                // Add dialog will open.
                break;
              case 'end':
                $(behavior.throbber).remove();
                window._layoutParagraphsSplit = undefined;
                // The user will continue from here.
                break;
            }
          }
        });
      }
    },

    getDuplicateButton(uuid) {
      return this.getComponent(uuid)?.querySelector('.lpb-duplicate');
    },

    getAddAfterButton(uuid) {
      return this.getComponent(uuid)?.querySelector('.lpb-btn--add.after');
    },

    getEditButton(uuid) {
      return this.getComponent(uuid)?.querySelector('.lpb-edit');
    },

    getComponent(uuid) {
      return document.querySelector(`.js-lpb-component[data-uuid="${uuid}"]`);
    },

    getNextComponentUuid(uuid) {
      return this.getComponent(uuid)?.nextElementSibling?.dataset?.uuid;
    },

  };

})(jQuery, Drupal, once);
